@@ -1,9 +1,9 @@
 """Serve files directly from the ContentsManager."""
 # Copyright (c) Jupyter Development Team.
 # Distributed under the terms of the Modified BSD License.
-import json
 import mimetypes
 from base64 import decodebytes
+from typing import List
 
 from tornado import web
 
@@ -14,7 +14,7 @@ from jupyter_server.utils import ensure_async
 AUTH_RESOURCE = "contents"
 
 
-class FilesHandler(JupyterHandler):
+class FilesHandler(JupyterHandler, web.StaticFileHandler):
     """serve files via ContentsManager
 
     Normally used when ContentsManager is not a FileContentsManager.
@@ -57,7 +57,7 @@ class FilesHandler(JupyterHandler):
 
         model = await ensure_async(cm.get(path, type="file", content=include_body))
 
-        if self.get_argument("download", False):
+        if self.get_argument("download", None):
             self.set_attachment_header(name)
 
         # get mimetype from filename
@@ -84,11 +84,9 @@ class FilesHandler(JupyterHandler):
             if model["format"] == "base64":
                 b64_bytes = model["content"].encode("ascii")
                 self.write(decodebytes(b64_bytes))
-            elif model["format"] == "json":
-                self.write(json.dumps(model["content"]))
             else:
                 self.write(model["content"])
             self.flush()
 
 
-default_handlers = []
+default_handlers: List[JupyterHandler] = []
